@@ -30,7 +30,11 @@ gboolean todo_style_changed =0;
 gboolean todo_list_autoresize = 1;
 gboolean todo_list_redraw_in_progess = 0;
 static void gncal_todo_init (GncalTodo *todo);
-static void modify_categories ( GtkWidget *widget, GnomeDialog *dialog );
+static gint gncal_todo_compare_date_func (GtkCList     *clist,
+					  gconstpointer ptr1,
+					  gconstpointer ptr2);
+
+static void modify_categories (GtkWidget *widget, GnomeDialog *dialog);
 
 
 
@@ -230,7 +234,7 @@ simple_todo_editor (GncalTodo *todo, iCalObject *ico)
 	gtk_box_pack_start ( GTK_BOX (pri_cat_box), pri_box, FALSE, FALSE, 0);
 	gtk_box_pack_start ( GTK_BOX (pri_cat_box), cat_box, FALSE, FALSE, 0);
 
-	pri_adj = gtk_adjustment_new (5.0, 1.0, 9.0, 1.0, 3.0, 0.0);
+	pri_adj = gtk_adjustment_new (ico->priority, 1.0, 9.0, 1.0, 3.0, 0.0);
 	pri_spin = gtk_spin_button_new (GTK_ADJUSTMENT(pri_adj), 0.0, 0);
 	gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (pri_spin), TRUE);
 	gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (pri_spin), FALSE);
@@ -313,15 +317,20 @@ add_todo (GncalTodo *todo)
 static void
 edit_todo (GncalTodo *todo)
 {
-	simple_todo_editor (todo, get_clist_selected_ico (todo->clist));
+	iCalObject *ico;
+	ico = get_clist_selected_ico (todo->clist);
+	if (ico)
+		simple_todo_editor (todo, ico);
 }
 
 static void
 delete_todo (GncalTodo *todo)
 {
-	gnome_calendar_remove_object (todo->calendar, get_clist_selected_ico (todo->clist));
-	save_default_calendar (todo->calendar);
-	
+	iCalObject *ico = get_clist_selected_ico (todo->clist);
+	if (ico) {
+		gnome_calendar_remove_object (todo->calendar, ico);
+		save_default_calendar (todo->calendar);
+	}
 }
 
 static void
@@ -373,9 +382,6 @@ modify_categories ( GtkWidget *widget, GnomeDialog *dialog)
 	gtk_entry_set_text ( cat_entry, categories_as_string ( categories ));
 }
 
-
-
-
 static void
 clist_row_selected (GtkCList *clist, gint row, gint column, GdkEventButton *event, GncalTodo *todo)
 {
@@ -443,6 +449,20 @@ init_column_sorting (GtkCList *clist)
 	gtk_clist_sort (clist);
 }
 
+static gint
+gncal_todo_compare_date_func (GtkCList *clist, gconstpointer ptr1, gconstpointer ptr2)
+{
+	iCalObject *ical1 = (iCalObject*)((GtkCListRow *) ptr1)->data;
+	iCalObject *ical2 = (iCalObject*)((GtkCListRow *) ptr2)->data;
+	
+	if (ical1->dtend > ical2->dtend)
+		return 1;
+	else if (ical1->dtend < ical2->dtend)
+		return -1;
+	else
+		return 0;
+}
+
 static void 
 todo_click_column (GtkCList *clist, gint column, gpointer data)
 {
@@ -461,6 +481,11 @@ todo_click_column (GtkCList *clist, gint column, gpointer data)
 		todo_current_sort_column = column;
 	}
 	
+	if (column == 1)
+		gtk_clist_set_compare_func (clist, gncal_todo_compare_date_func);
+	else
+		gtk_clist_set_compare_func (clist, NULL);
+
 	gtk_clist_sort (clist); 
   
 	/*
@@ -468,9 +493,19 @@ todo_click_column (GtkCList *clist, gint column, gpointer data)
 	 * click twice
 	 */
 
-	gnome_config_set_int("/calendar/Todo/sort_column", todo_current_sort_column);
-	gnome_config_set_int("/calendar/Todo/sort_type", todo_current_sort_type);
-	gnome_config_sync();
+	gnome_config_set_int ("/calendar/Todo/sort_column", todo_current_sort_column);
+	gnome_config_set_int ("/calendar/Todo/sort_type", todo_current_sort_type);
+	gnome_config_sync ();
+}
+
+static void
+clist_unselect_row (GtkCList *clist, gint row, gint column, GdkEventButton *event, gpointer data)
+{
+	GncalTodo *todo = data;
+	if (!clist->selection) {
+		gtk_widget_set_sensitive (todo->edit_button, FALSE);
+		gtk_widget_set_sensitive (todo->delete_button, FALSE);
+	}
 }
 
 static void
@@ -480,18 +515,12 @@ gncal_todo_init (GncalTodo *todo)
 	GtkWidget *sw;
 	GtkWidget *hbox;
 	gchar *titles[5] = {
-	    N_("Summary"),
-	    N_("Due Date"),
-	    N_("Priority"),
-	    N_("Time Left"),
-	    N_("Categories")
+		_("Summary"),
+		_("Due Date"),
+		_("Priority"),
+		_("Time Left"),
+		_("Categories")
 	};
-	char *tmp[5];
-	tmp[0] = _(titles[0]);
-	tmp[1] = _(titles[1]);
-	tmp[2] = _(titles[2]);
-	tmp[3] = _(titles[3]);
-	tmp[4] = _(titles[4]);
 
 	gtk_box_set_spacing (GTK_BOX (todo), 5);
 
@@ -510,10 +539,10 @@ gncal_todo_init (GncalTodo *todo)
 	gtk_widget_show (sw);
 
 
-	w = gtk_clist_new_with_titles(5, tmp);
+	w = gtk_clist_new_with_titles (5, titles);
 
 	todo->clist = GTK_CLIST (w);
-	gtk_clist_set_selection_mode (todo->clist, GTK_SELECTION_BROWSE);
+	gtk_clist_set_selection_mode (todo->clist, GTK_SELECTION_SINGLE);
 
 	gtk_signal_connect (GTK_OBJECT (todo->clist), "select_row",
 			    (GtkSignalFunc) clist_row_selected,
@@ -525,6 +554,10 @@ gncal_todo_init (GncalTodo *todo)
 	gtk_signal_connect (GTK_OBJECT (todo->clist), "click_column",
 			  (GtkSignalFunc) todo_click_column, NULL);
 
+	gtk_signal_connect (GTK_OBJECT (todo->clist), "unselect-row",
+			    (GtkSignalFunc) clist_unselect_row,
+			    todo);
+
 	gtk_container_add (GTK_CONTAINER (sw), w);
 	gtk_widget_show (w);
 
@@ -716,10 +749,10 @@ insert_in_clist (GncalTodo *todo, iCalObject *ico)
 
 	text[0] =  ico->summary;
 
-	if(todo_show_time_remaining) {
-	  memset(time_remaining_buffer, 0, 100);
-	  /* we need to make a string that represents the amount of time remaining
-	     before this task is due */
+	if (todo_show_time_remaining) {
+	  memset (time_remaining_buffer, 0, 100);
+	  /* we need to make a string that represents the amount of
+	     time remaining before this task is due */
 	  
 	  /* for right now all I'll do is up to the hours. */
 	  time_remain = (ico->dtend - time(NULL));
@@ -762,31 +795,31 @@ insert_in_clist (GncalTodo *todo, iCalObject *ico)
 		seconds = time_remain;
 	      }
 
-	    switch(time_remaining_form) 
+	    switch (time_remaining_form) 
 	      {
 	      case TODO_ITEM_REMAINING_WEEKS:
-		snprintf(time_remaining_buffer, 100, "%d %s %d %s", weeks,
-			 (weeks > 1) ? _("Weeks") : _("Week"),
-			 days, (days > 1) ? _("Days") : _("Day"));
+		snprintf (time_remaining_buffer, 100, "%d %s %d %s", weeks,
+			  (weeks > 1) ? _("Weeks") : _("Week"),
+			  days, (days > 1) ? _("Days") : _("Day"));
 		break;
 	      case TODO_ITEM_REMAINING_DAYS:
-		snprintf(time_remaining_buffer, 100, "%d %s %d %s", days,
-			 (days > 1) ? _("Days") : _("Day"),
-			 hours, (hours > 1) ? _("Hours") : _("Hour"));
+		snprintf (time_remaining_buffer, 100, "%d %s %d %s", days,
+			  (days > 1) ? _("Days") : _("Day"),
+			  hours, (hours > 1) ? _("Hours") : _("Hour"));
 		break;
 	      case TODO_ITEM_REMAINING_HOURS:
-		snprintf(time_remaining_buffer, 100, "%d %s %d %s", hours,
-			 (hours > 1) ? _("Hours") : _("Hour"),
-			 minutes, (minutes > 1) ? _("Minutes") : _("Minute"));
+		snprintf (time_remaining_buffer, 100, "%d %s %d %s", hours,
+			  (hours > 1) ? _("Hours") : _("Hour"),
+			  minutes, (minutes > 1) ? _("Minutes") : _("Minute"));
 		break;
 	      case TODO_ITEM_REMAINING_MINUTES:
-		snprintf(time_remaining_buffer, 100, "%d %s %d %s", minutes,
-			 (minutes > 1) ? _("Minutes") : _("Minute"),
-			 seconds, (seconds > 1) ? _("Seconds") : _("Second"));
+		snprintf (time_remaining_buffer, 100, "%d %s %d %s", minutes,
+			  (minutes > 1) ? _("Minutes") : _("Minute"),
+			  seconds, (seconds > 1) ? _("Seconds") : _("Second"));
 		break;
 	      case TODO_ITEM_REMAINING_SECONDS:
-		snprintf(time_remaining_buffer, 100, "%d %s", seconds,
-			 (seconds > 1) ? _("Seconds") : _("Second")); 
+		snprintf (time_remaining_buffer, 100, "%d %s", seconds,
+			  (seconds > 1) ? _("Seconds") : _("Second")); 
 		break;
 	      }
 	    text[3] = g_strdup(time_remaining_buffer);
