@@ -523,6 +523,10 @@ GnomeUIInfo sortradios[] = {
 	 gnomecard_sort_cards, (gpointer) COLTYPE_FULLNAME, NULL,
 	GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
 	
+	{GNOME_APP_UI_ITEM, N_("By Last Name"), "",
+	 gnomecard_sort_cards, (gpointer) COLTYPE_LASTNAME, NULL,
+	GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
+	
 	{GNOME_APP_UI_ITEM, N_("By E-mail"), "",
 	 gnomecard_sort_cards, (gpointer) COLTYPE_EMAIL, NULL,
 	GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
@@ -703,6 +707,7 @@ void gnomecard_init(void)
 	GtkWidget *scrollwin;
         GnomeAppBar *appbar;
 	GtkWidget *canvas, *hbox;
+	gchar *s;
 	gint ncol, i, x, y;
 
 	/* hard coded column headers */
@@ -750,14 +755,19 @@ void gnomecard_init(void)
 
 	/* card display layout */
 	ncol=gnome_config_get_int("/GnomeCard/CardDisplay/ncols=0");
+	s = gnome_config_get_string("/GnomeCard/CardDisplay/SortType=COLTYPE_CARDNAME");
+	gnomecard_sort_col = getColumnTypeFromTypeName(s);
+	if (gnomecard_sort_col == COLTYPE_END)
+		gnomecard_sort_col = COLTYPE_CARDNAME;
+
 	if (!ncol) {
 	    gnomecard_list = GTK_CLIST(gnomecardCreateCardListDisplay(defaulthdrs));
 	} else {
+	    gchar path[200];
+
+	    
 	    hdrs = g_new0(ColumnType, ncol+1);
 	    for (p=hdrs, i=0; i<ncol; i++) {
-		gchar path[200];
-		gchar *s;
-
 		snprintf(path, sizeof(path),
 			 "/GnomeCard/CardDisplay/Column%0d=", i);
 		s=gnome_config_get_string(path);
@@ -792,7 +802,7 @@ void gnomecard_init(void)
 	gtk_container_add(GTK_CONTAINER(scrollwin), canvas);
 	gtk_widget_show(canvas);
 	gtk_widget_show(scrollwin);
-	gtk_box_pack_start(GTK_BOX(hbox), scrollwin, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), scrollwin, TRUE, TRUE, 0);
 /*	gtk_paned_pack2 (GTK_PANED (hpaned), scrollwin, TRUE, TRUE);*/
 
 	/* no cards yet */
@@ -817,14 +827,12 @@ void gnomecard_init(void)
 	menu_prev = gomenu[1].widget;
 	menu_next = gomenu[2].widget;
 	menu_last = gomenu[3].widget;
-
-	gnomecard_sort_col = COLTYPE_CARDNAME;
 	
 	gnomecard_init_defaults();
 	gnomecard_set_changed(FALSE);
 	
-	if (! gnomecard_append_file(gnomecard_fname))
-	  gnomecard_set_curr(NULL);
+	if (! gnomecard_open_file(gnomecard_fname))
+          gnomecard_set_curr(NULL);
 
 	gtk_widget_show(GTK_WIDGET (gnomecard_window));
 }
